###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Erik Enge <erik@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.12 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

from Globals import HTMLFile
from ZPublisher.HTTPRequest import html_quote
import ZODB
import string
import types
import html_widgets

class base_item:
	"""Base class we use for easy access to often used methods."""

	meta_types = ()
	help_name = ''

	__ac_permissions__ = () # !FIXME!

	base_delete = HTMLFile('base_delete', globals())
	base_edit = HTMLFile('base_edit', globals())
	base_edit_display = HTMLFile('base_edit_display', globals())

	local_frame = HTMLFile('base_local_frame', globals())

	counter = 0

	use_local_navigation = 1
	use_local_preferences = 1
	use_breadcrumbs = 1


	def get_navigation_meta_types(self):
		"""Returns relevant meta types for navigation."""

		return self.meta_types


	def get_property(self, prop_name):
		"Return true if self._properties contains prop_name."
		for x in range(len(self._properties)): 
			if self._properties[x]['id'] == prop_name:
				return 1
			'''
			for key in self._properties[x].keys():
				if str(prop_name) == str(self._properties[x]['id']):
					return 1
			'''
		return 0

	def name_title_or_id(self):
		"Returns either job_title, firstname + surname, name, title, meta_type or id - in that order."
		if self.get_property('job_title') == 1:
			job_title=self.get_job_title()
		else:
			job_title=''
		
		if self.get_property('name') == 1:
			name=self.name
		else:
			name=''

		if self.get_property('firstname') == 1:
			firstname=self.firstname
			surname=self.surname
		else:
			firstname=''
			surname=''

		title=self.title
		if callable(title):
			title=title()

		id=self.id
		if callable(id):
			id=id()

		if (len(job_title) > 0 and self.meta_type != 'Candidate'):
			return job_title +' / '+str(self.id)
		elif len(firstname) > 0:
			return firstname +' '+ surname
		elif len(name) > 0:
			return name
		elif len(title) > 0:
			return title 
		else:
			return id


	def get_help_html(self):
		"""Returns a submission button used for accessing help."""

		if self.help_name:

			return """<a href="%s" target="_new">Help</a>""" % \
				("/Control_Panel/Products/%s/Help/%s" % (self.product_name, self.help_name))

		else:

			return ""


	def local_navigation(self, *args, **kw):
		"""Returns a set of widgets for adding objects."""

		if self.REQUEST.has_key('skip_local_navigation') and\
			self.REQUEST['skip_local_navigation']:

			return ""

		if string.find(self.REQUEST.URL0, 'manage_add') > -1 or \
			 string.find(self.REQUEST.URL0, 'manage_edit') > -1:

			return ""

		actions = []
		meta_types = list(self.get_navigation_meta_types())

		if getattr(self.aq_base, 'use_all_meta_types', None):

			meta_types.extend(list(self.all_meta_types()))

		for subclass in meta_types:

			actions.append("""<option value="%s">%s</option>""" % \
						(subclass['action'], subclass['name']))

		else:

			if actions:

				action_html = """<select name=":method">%s</select>""" % \
					string.join(actions)
				action_html = action_html + """<input type="submit" value=" Add ">"""

			else:

				action_html = ""

		if getattr(self.aq_base, 'deleteable_object', None):

			delete_html = """<input type="submit" value=" Delete " name="delete:method">"""

		else:

			delete_html = ""

		if getattr(self.aq_base, 'editable_object', None):

			edit_html = """<input type="submit" value=" Edit " name="base_edit:method">"""

		else:

			edit_html = ""

		help_html = self.get_help_html()


		return """<form action="%s">""" % self.relative_url() + \
			action_html + delete_html + edit_html + help_html + """</form>"""


	def get_preferences_url(self):
		"""Returns the URL used to manage product instance preferences."""

		return self.relative_url() + '/base_edit'


	def copy(self):
		"""Returns a copy of self."""

		return self.getParentNode().manage_copyObjects(self.id)


	def paste(self, copy_data, new_id=None):
		"""Pastes the copy into self."""

		self.manage_pasteObjects(cb_copy_data=copy_data)


	def cut(self):
		"""Cuts self from containing object."""

		return self.getParentNode().manage_cutObjects(self.id)


	def delete(self, RESPONSE=None):
		"""Deletes self."""

		self.getParentNode().manage_delObjects(self.id)

		if RESPONSE is not None:

			RESPONSE.redirect(self.getParentNode().relative_url())


	def navigation(self, separator_html=' &gt;&gt; ',):
		"""Renders breadcrumb navigation."""

		path = self.REQUEST['PARENTS']
		html_path = []

		for element_index in range(len(path)):

			element = path[element_index]

			if hasattr(element.aq_base, 'stop_breadcrumbs') and \
				element.stop_breadcrumbs:

				# If we're told that this is as far as it goes..

				break

			if hasattr(element.aq_base, 'name_title_or_id') and \
				callable(element.name_title_or_id):

				title = element.name_title_or_id()

			else:

				title = element.title_or_id()

			title = html_quote(title)

			html_path.append("""<a href="%s">%s</a>""" % \
						('../' * element_index, title))

		html_path.reverse()

		return """<span class="navigationbaritem">%s</span>""" % \
				string.join(html_path, separator_html)

	navigation_bar = navigation


	def footer(self, context):
		""

		return ""


	def product_installed(self, product_name):
		"""A method that returns a truth value if given product is installed."""

		if product_name in self.Control_Panel.Products.objectIds():

			return 1

		else:

			return 0


	def get_nearest_object_by_meta_type(self, meta_type):
		"""Returns the nearest instance with given meta type."""

		instance = None

		reference = self

		while 1:

			if reference == None:

				# If we've come to the end of the line

				break

			try:

				instances = \
					reference.objectValues(meta_type)

			except AttributeError:

				# We've come to the Application instance

				break

			if instances:

				# If we found objects on the way

				instance = \
					instances[0]

				break

			reference = reference.getParentNode()

		return instance


	def get_all_objects_by_meta_type(self, meta_type):
		"""Returns all objects in acquisition path of given meta type."""

		return map(lambda x: x[-1],
				self.ZopeFind(self, obj_metatypes=[meta_type], search_sub=1))

		# ?FIXME?

		"""
		instances = []

		reference = self

		while 1:

			instances = instances + reference.objectValues(meta_type)

			reference = self.getParentNode()

			if reference is None:

				break

		return instances
		"""


	def sneak_peak(self):
		"""Brings up a useful display of the object."""

		# Note:
		#
		# This makes all the properties defined available to
		# all users with access to the object.

		# Thanks to Steve Alexander for the aq_base.__class__.__name__
		# tip.

		return_string = "<class %s at %s" % (self.aq_base.__class__.__name__,
							self.relative_url())

		for property in self._properties:

			property_id = property['id']
			value = eval("self.aq_base.%s" % property_id)

			return_string = return_string + "\n\t%s (%s): %s" % \
				(property_id, type(value), value)

		else:

			return_string = return_string + '\n>'

		return return_string


	def manage_delete(self, ids=[], REQUEST=None, RESPONSE=None):
		"""Deletes given ids."""

		if type(ids) is type(''):

			ids = [ids]

		self.manage_delObjects(ids)

		if REQUEST is not None:

			if REQUEST.has_key('__referrer'):

				RESPONSE.redirect(REQUEST['__referrer'])

			else:


				RESPONSE.redirect(self.absolute_url())


	def redirect_next_id(self, REQUEST, RESPONSE):
		"""Returns the 'next' object id."""

		object_id = REQUEST['__object_id']
		
		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			next = objectIds[objectIds.index(object_id) + 1]

		except IndexError:

			# We've got the last id, wrapping around to the first one.

			next = objectIds[0]

		RESPONSE.redirect(reference.absolute_url() + '/' + next)


	def redirect_previous_id(self, REQUEST, RESPONSE):
		"""Returns the 'previous' object id."""

		object_id = REQUEST['__object_id']

		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			previous = objectIds[objectIds.index(object_id) - 1]

		except ValueError:

			# We've got the first id in the list, wrap it.

			previous = objectIds[-1]

		RESPONSE.redirect(reference.absolute_url() + '/' + previous)


	def export_as_understandable_xml(self):
		"""Exports the instance in a 'pure' xml format."""

		# We want this to export variables defined in the _properties
		# dictionary only.

		# TODO !

		raise NotImplemented


	def manage_delete_all(self, REQUEST=None, RESPONSE=None):
		"""Method that deletes all messages in folder."""

		if REQUEST is not None:

			if REQUEST.has_key('confirm') and \
				REQUEST['confirm']:

				pass

			else:

				RESPONSE.redirect(self.absolute_url())
				return

		self.manage_delete(ids=self.objectIds())

		RESPONSE.redirect(self.absolute_url())


	def relative_url(self):
		"""Returns the URL relative to the hostname and port."""

		return string.join(self.getPhysicalPath(), '/')


	def resolve_relative_url(self, relative_url):
		"""Method that resolves a URL assuming it is in the local namespace."""

		return self.restrictedTraverse(path=relative_url)


	def get_last_modified_by(self):
		"""Returns the username of the user which last modified this instance."""

		undoable_transactions = self.undoable_transactions()

		if undoable_transactions:

			return undoable_transactions.pop()['user_name']

		else:

			return self.creator


	def get_parent_node_title(self):
		"""Returns the title of the parent node."""

		return self.getParentNode().title


	def get_parent_node_relative_url(self):
		"""Returns the relative url of the parent node."""

		return self.getParentNode().relative_url()


	def get_parent_node_id(self):
		"""Returns the id of the parent node, used for ZCatalog indexing."""

		return self.getParentNode().id


	def get_html_properties_listing(self, property_sheet=None):
		"""Returns a list of mappings, suitable for iterations."""

		html_properties = []
		html_properties_mapping = self.get_html_properties_mapping(property_sheet=
						property_sheet)

		for property_id in html_properties_mapping['html_properties_order']:

			html_properties.append(html_properties_mapping[property_id])

		return html_properties


	def get_html_properties_mapping(self, property_sheet=None):
		"""Returns the HTML needed to represent and manipulate properties with HTML widgets."""

		string_capwords = string.capwords
		string_replace = string.replace

		html_textarea = html_widgets.textarea
		html_select = html_widgets.select
		html_select_option = html_widgets.select_option
		html_radio_collection = html_widgets.radio_collection
		html_radio = html_widgets.radio
		html_text = html_widgets.text

		html = {'html_properties_order':[]}

		properties = []

		if property_sheet is not None:

			property_sheet = self.property_sheets[property_sheet]

			for property in self._properties:

				if property['id'] in property_sheet:

					properties.append(property)

		else:

			properties = self._properties

		for property in properties:

			property_id = property['id']
			property_value = getattr(self, property['id'])
			print property_id, property_value, type(property_value)
			property_type = property['type']
			property_html_name = property_id + ';' + property_type

			html['html_properties_order'].append(property_id)

			if property.has_key('select_widget'):

				select_widget = property['select_widget']

				try:

					html_widget_multiple = property['multiple']

				except KeyError:

					html_widget_multiple = 0

			else:

				select_widget = ""

				if property.has_key('boolean_widget'):

					boolean_widget = property['boolean_widget']

				else:

					boolean_widget = ""

			if property.has_key('rendered_id'):

				rendered_id = property['rendered_id']

			else:

				rendered_id = string_capwords(string_replace(property_id, '_', ' '))

			if property_type == 'text':

				html_widget = html_textarea(name=property_html_name, value=property_value)

			elif property_type == 'lines':

				html_widget = html_textarea(name=property_html_name,
						value=string.join(property_value, '\n'),
						rows=5,
						cols=30)

			elif select_widget:

				select_widget = getattr(self, select_widget)

				if callable(select_widget):

					options = select_widget()

				else:

					options = select_widget

				html_widget = html_widgets.select(name=property_html_name,
									multiple=html_widget_multiple)

				# Dictionaries are always messy..
				# Should we add some sort of logic here
				# to prevent sorting of keys?

				option_keys = options.keys()
				option_keys.sort()

				for option_key in option_keys:

					option = html_select_option(option_key, options[option_key])

					if option_key == property_value:

						option.select()

					html_widget.add_option(option)


			elif property_type == 'boolean':

				boolean_widget = getattr(self, boolean_widget)

				if callable(boolean_widget):

					options = boolean_widget()

				else:

					options = boolean_widget

				html_widget = html_radio_collection(name=property_html_name)

				# Bleh

				option_keys = options.keys()
				option_keys.sort()

				for option_key in option_keys:

					value = options[option_key]

					if option_key == property_value:

						html_widget.add_option(
							value=option_key,
							displayed_value=value,
							checked=1)

					else:

						html_widget.add_option(
							value=option_key,
							displayed_value=value)

			else:

				if property_type in ('int', 'float'):

					size = 10

				else:

					size = None

				html_widget = html_text(name=property_html_name, value=property_value, size=size)
					
			html[property_id] = \
				{
					'html_widget':html_widget,
					'rendered_id':rendered_id
				}

		return html


	def base_edit_properties(self, REQUEST):
		"""Update properties, with a nice redirect."""

		# This method assumes that all properties defined
		# are settable by anyone with access to the object.

		# FIXME
		# setattr = setattr

		if REQUEST.has_key('__ignore_keyerror') and REQUEST['__ignore_keyerror']:

			for property in self._properties:

				try:

					exec("self.%s = REQUEST.form['%s']" % (property['id'], property['id']))

				except KeyError:

					# Hrm, why is the variable at times unnacessible from the form,
					# and sometimes from the REQUEST itself?  Even when it's there.

					try:

						exec("self.%s = REQUEST.%s" % (property['id'], property['id']))

					except KeyError:

						pass

		else:

			for property in self._properties:

				try:

					exec("self.%s = REQUEST.form['%s']" % (property['id'], property['id']))

				except KeyError:

					exec("self.%s = REQUEST.%s" % (property['id'], property['id']))

		try:

			self.reindex_object()

		except AttributeError:

			pass

		if REQUEST.has_key('__referrer'):

			REQUEST.RESPONSE.redirect(REQUEST['__referrer'])

		else:

			REQUEST.RESPONSE.redirect(self.relative_url() + '/base_edit')


	def partial_reindex(self, catalog, indexes, uid):
		"""Does a partial reindex of the object, based on the indexes specified."""

		pass


	def local_frame_navigation(self):
		"""Wrapper for local_navigation."""

		return self.local_navigation()
