###########################################################################
#
#   Copyright (c) 2000, 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000, 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.1 $


from Globals import HTMLFile
import exceptions


def pad_integer(integer, size=2):
	"""Function that pads an integer to a string with the given size."""

	# Mad props to Steve Alexander for this sweet patch

	return ('%0'+`size`+'d') % int(integer)


def format_time(value):
	"""Returns a HH:MM style formatted representation of the time."""

	value = int(value)

	hours = value / (60*60)
	minutes = value % (60*60) / 60

	return pad_integer(hours) + ':' + pad_integer(minutes)


def compile_time(hour, minute):
	"""Returns a compiled integer value."""

	hour = int(hour)
	minute = int(minute)

	return ((hour * 60 * 60) + (minute * 60))


def make_hour_minutes_select(hour=None, minute=None, name=None, seconds=None, minute_step=5,
				selected_hour=None, selected_minute=None, integer=1):
	"""Returns a <SELECT> widget."""

	if selected_minute or selected_hour:

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = selected_hour)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1,
					selected = selected_minute, step=minute_step)

	elif seconds:

		seconds = int(seconds)

		hour = seconds / (60*60)
		minute = (seconds%(60*60)/60)

		if selected_minute is None and \
			selected_hour is None:

			selected_hour = hour
			selected_minute = minute

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = selected_hour)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1,
					selected=selected_minute, step=minute_step)

	else:

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = 12)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1, selected = 0,
					step=minute_step)

	return tmp


def make_range_select(name, start, stop, step=1, selected=None, pad=None, integer=1):
	"""Returns a <SELECT> widget."""

	if integer:

		tmp = """<select name="%s:int">""" % name

	else:

		tmp = """<select name="%s">""" % name

	option_dict = {}

	if selected:

		for i in range(start, stop, step):

			if pad:

				displayed_value = pad_integer(i)

			else:

				displayed_value = i

			if i == selected:

				tmp = tmp + """<option value="%s" selected>%s</option>\n""" % (i, displayed_value)

			else:
				
				tmp = tmp + """<option value="%s">%s</option>\n""" % (i, displayed_value)
			

	else:

		for i in range(start, stop, step):

			if pad:

				displayed_value = pad_integer(i)

			else:

				displayed_value = i

			tmp = tmp + """<option value="%s">%s</option>\n""" % (i, displayed_value)

	tmp = tmp + """</select>"""

	return tmp


def format_size(bytes, details=None):
	"""Renders a human-readable display of the size."""

	return_string = ""

	if bytes > 1024*1024:

		return_string = return_string + \
			"%.2f MB" % (bytes / 1024.0 / 1024)

	else:

		return_string = return_string + \
			"%.2f KB" % (bytes / 1024.0)

	if details:

		return_string = return_string + " (%s bytes)" % bytes

	return return_string	


product_base_manage_theme = HTMLFile('product_base_manage_theme', globals())

def product_base_update_theme(self, REQUEST, RESPONSE):
	"""Updates the theme settings."""

	self.color_body0 = REQUEST['color_body0']
	self.color_body1 = REQUEST['color_body1']
	self.color_header0 = REQUEST['color_header0']
	self.color_header1 = REQUEST['color_header1']
	self.color_header_description0 = REQUEST['color_header_description0']
	self.color_sequence_even0 = REQUEST['color_sequence_even0']
	self.color_sequence_odd0 = REQUEST['color_sequence_odd0']
	self.table_cellpadding0 = REQUEST['table_cellpadding0']
	self.table_cellspacing0 = REQUEST['table_cellspacing0']
	self.table_border0 = REQUEST['table_border0']

	#RESPONSE.redirect(self.relative_url() + '/product_base_manage_theme')
	

def default_installation(instance,
		install_default_html=None,
		install_default_theme=None,
		install_default_id_generator=None):
	"""Installs product specific 'add-ons'."""

	# ?FIXME?
	#
	# We should move this to be a method of product_base_item instead.

	try:

		from Products import ThreadSafeCounter

	except ImportError:

		ThreadSafeCounter = None

	if install_default_html:

		# ?FIXME?
		from OFS.DTMLMethod import addDTMLMethod

		addDTMLMethod(instance, 'standard_html_header',
				'standard_html_header',
				 open('%s/Products/%s/standard_html_header.dtml' % \
					(SOFTWARE_HOME, instance.product_name)).read())
									  
		addDTMLMethod(instance, 'standard_html_footer',
				'standard_html_footer',
				open('%s/Products/%s/standard_html_footer.dtml' % \
					(SOFTWARE_HOME, instance.product_name)).read())

		addDTMLMethod(instance, 'standard_css',
				'standard_css',
				open('%s/Products/%s/standard_css.dtml' % \
					(SOFTWARE_HOME, instance.product_name)).read())

	if install_default_theme:

		instance.color_id0 = 'lightblue'
		instance.color_body0 = 'white'
		instance.color_body1 = 'lightgrey'
		instance.color_action1 = 'lightblue'
		instance.color_action0 = 'grey'
		instance.color_header0 = 'wheat'
		instance.color_header1 = 'wheat'
		instance.color_header_description0 = '#abcdef'
		instance.color_sequence_even0 = '#cccccc'
		instance.color_sequence_odd0 = '#aeaeae'
		instance.table_cellpadding0 = 4
		instance.table_cellspacing0 = 0
		instance.table_border0 = 0
		instance.text_input_size = 50

		instance.product_base_manage_theme = product_base_manage_theme
		instance.product_base_update_theme = product_base_update_theme

	if install_default_id_generator:

		if ThreadSafeCounter is None:

			raise exceptions.system_error.product_not_installed(
				name='ThreadSafeCounter')

		ThreadSafeCounter.thread_safe_counter.manage_add_thread_safe_counter(instance)

